'use strict';

// process.stdout.isTTY
var form = require('terminal-forms.js').create(process.stdin, process.stdout);
var OAUTH3 = require('../oauth3.node.js');
// TODO change to ._hooks
OAUTH3._hooks = require('../oauth3.node.storage.js');
// directives = { get, set }
// sessions = { get, set }
OAUTH3.hooks.directives._set = require('../oauth3.node.storage.js').directives._set;
OAUTH3.hooks.session._get = require('../oauth3.node.storage.js').session._get;
OAUTH3.hooks.session._set = require('../oauth3.node.storage.js').session._set;
OAUTH3._hooks.directives.get = require('../oauth3.node.storage.js').directives._get;
OAUTH3._hooks.directives.set = require('../oauth3.node.storage.js').directives._set;
OAUTH3._hooks.session.get = require('../oauth3.node.storage.js').session._get;
OAUTH3._hooks.session.set = require('../oauth3.node.storage.js').session._set;

var url = require('url');
//console.log('stdin tty', process.stdin.isTTY);
//console.log('stdout tty', process.stdout.isTTY);

form.ask({ label: "What's your OAuth3 Provider URL? ", type: 'url' }).then(function (urlResult) {
  var urlObj = url.parse(urlResult.result || urlResult.input);
  // TODO get unique client id for bootstrapping app
  var oauth3 = OAUTH3.create(urlObj);
  var providerPromise = oauth3.setProvider(urlObj.host + urlObj.pathname);

  function getCurrentUserEmail() {
    return form.ask({ label: "What's your email (or cloud mail) address? ", type: 'email' }).then(function (emailResult) {
      // TODO lookup uuid locally before performing loginMeta
      // TODO lookup token locally before performing loginMeta / otp
      return providerPromise.then(function () {
        return OAUTH3.authn.loginMeta(oauth3._providerDirectives, { email: emailResult.input }).then(function (/*result*/) {
          return emailResult.input;
        }, function (/*err*/) {
          // TODO require hashcash to create user account
          function confirmCreateAccount() {
            // TODO directives should specify private (invite-only) vs internal (request) vs public (allow) accounts
            return form.ask({
              label: "We don't recognize that address. Do you want to create a new account? [Y/n] "
            , type: 'text' // TODO boolean with default Y or N
            }).then(function (result) {
              if (!result.input) {
                result.input = 'Y';
              }

              result.input = result.input.toLowerCase();

              if ('y' !== result.input) {
                return getCurrentUserEmail();
              }

              return emailResult.input;
            });
          }

          return confirmCreateAccount();
        });
      });
    });
  }

  return getCurrentUserEmail().then(function (email) {
    // TODO skip if token exists locally
    form.println("Sending login code to '" + email + "'...");
    return OAUTH3.authn.otp(oauth3._providerDirectives, { email: email }).then(function (otpResult) {
      return form.ask({
        label: "What's your login code? "
      , help: "(it was sent to '" + email + "' and looks like 1234-5678-9012)"
        // onkeyup
        // ondebounce
        // onchange
        // regexp // html5 name?
      , onReturnAsync: function (rs, ws, input/*, ch*/) {
          var formatted = input.toLowerCase().replace(/[^\d]+/g, '');

          if (12 !== formatted.length) {
            return form.PromiseA.reject(new Error("invalid code please try again in the format xxxx-yyyy-zzzz"));
          }

          formatted = formatted.match(/.{4,4}/g).join('-');

          if (14 !== formatted.split('').length) {
            return form.PromiseA.reject(new Error("invalid code '" + formatted + "', please try again xxxx-yyyy-zzzz"));
          }

          var data = {
            username: email
          , username_type: 'email'
          , client_id: OAUTH3.uri.normalize(oauth3._providerDirectives.issuer)
          , client_uri: OAUTH3.uri.normalize(oauth3._providerDirectives.issuer)
          , otp_code: formatted
          , otp_uuid: otpResult.data.uuid
          };

          // returns session instead of input
          var colors = require('colors');
          form.setStatus(colors.dim("authenticating with server..."));
          return OAUTH3.authn.resourceOwnerPassword(oauth3._providerDirectives, data);
        }
      }).then(function (results) {
        var session = results.result;

        form.println('session:');
        form.println(session);
      });
    });
  });
});
